@@ -75,7 +75,7 @@ void Traits::ImageEffects::Blur::setStrength(qreal strength)
     m_backingStoreCache = {};
 }
 
-QImage Traits::ImageEffects::Blur::image(std::function<QImage()> getImage, QRectF rect, qreal dpr) const
+QImage Traits::ImageEffects::Blur::image(const std::function<QImage()> &getImage, QRectF rect, qreal dpr) const
 {
     if ((m_backingStoreCache.isNull() //
          || m_backingStoreCache.devicePixelRatio() != dpr //
@@ -85,6 +85,8 @@ QImage Traits::ImageEffects::Blur::image(std::function<QImage()> getImage, QRect
         if (m_backingStoreCache.isNull()) {
             return m_backingStoreCache;
         }
+        // RGBA is better for use with OpenCV
+        m_backingStoreCache.convertTo(QImage::Format_RGBA8888_Premultiplied);
         auto mat = QtCV::qImageToMat(m_backingStoreCache);
         // Below this, the effect is nearly invisible.
         static const qreal min = 0.5;
@@ -125,7 +127,7 @@ void Traits::ImageEffects::Pixelate::setStrength(qreal strength)
     m_backingStoreCache = {};
 }
 
-QImage Traits::ImageEffects::Pixelate::image(std::function<QImage()> getImage, QRectF rect, qreal dpr) const
+QImage Traits::ImageEffects::Pixelate::image(const std::function<QImage()> &getImage, QRectF rect, qreal dpr) const
 {
     if ((m_backingStoreCache.isNull() //
          || m_backingStoreCache.devicePixelRatio() != dpr //
