@@ -14,7 +14,7 @@
 class VideoFormatModel : public QAbstractListModel
 {
     Q_OBJECT
-    Q_PROPERTY(int count READ rowCount CONSTANT FINAL)
+    Q_PROPERTY(int count READ rowCount NOTIFY countChanged FINAL)
 public:
     VideoFormatModel(VideoPlatform::Formats formats, QObject *parent = nullptr);
 
@@ -23,6 +23,8 @@ public:
         ExtensionRole = Qt::UserRole + 2,
     };
 
+    void setFormats(VideoPlatform::Formats formats);
+
     int indexOfFormat(VideoPlatform::Format format) const;
 
     QHash<int, QByteArray> roleNames() const override;
